/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.farm;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.EnumSet;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.farm.model.Farm;
import nl.cloudfarming.client.farm.model.FarmManager;
import nl.cloudfarming.client.model.AbstractEntityManager;
import nl.cloudfarming.client.model.AgroURI;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import nl.cloudfarming.client.model.ItemIdType;
import org.netbeans.api.project.Project;
import org.netbeans.spi.project.ui.support.DefaultProjectOperations;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.util.Exceptions;
import org.openide.util.lookup.ServiceProvider;

/**
 * Stateless
 *
 * @author Timon Veenstra
 */
@ServiceProvider(service = FarmManager.class)
public class FarmManagerImpl extends AbstractEntityManager<Farm> implements FarmManager {

    private static final Set<ItemIdType> SUPPORTED_TYPES = EnumSet.of(ItemIdType.FRM);
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.farm");

    @Override
    public Set<ItemIdType> getSupportedTypes() {
        return SUPPORTED_TYPES;
    }

    /**
     * Load a Farm from a file object
     *
     *
     * @param fileObject
     * @return
     */
    Farm load(Project project, FileObject fileObject) {
        Farm farm = null;
        try (ObjectInputStream ois = new ObjectInputStream(fileObject.getInputStream())) {
            farm = (Farm) ois.readObject();
            // FIXME: postpone farm caching until project is actually opened;
            //   load is already called when the project folder is shown in the 'open project' dialog.
            addToCache(farm);
        } catch (ClassNotFoundException | IOException ex) {
            LOGGER.log(Level.SEVERE, "Unable to load farm from FileObject {0} {1}", new Object[]{fileObject.getName(), ex.getMessage()});
        }
        return farm;
    }

    @Override
    public Project findFarmProject(AgroURI agroURI) {
        return EnterpriseIdProvider.ProjectFinder.find(agroURI);
    }

    void newFarm(Farm farm, FileObject location) {
        save(farm, location);
    }

    private void save(Farm farm, FileObject projectRoot) {
        try {
            FileObject fo = projectRoot.getFileObject(FarmProject.FARM_FILE);
            if (fo == null) {
                fo = projectRoot.createData(FarmProject.FARM_FILE);
            }

            try (ObjectOutputStream oos = new ObjectOutputStream(fo.getOutputStream())) {
                oos.writeObject(farm);
                addToCache(farm);
            }

        } catch (IOException ex) {
            Exceptions.printStackTrace(ex);
            throw new RuntimeException("Failed to create farm " + ex.getMessage());
        }
    }

    /**
     * Save a farm object
     *
     * <p> Warning <br/> The save method for farms cannot be used to create new
     * farms. New farms should always be created with the {@link FarmManagerImpl#newFarm(nl.cloudfarming.client.farm.model.Farm, org.openide.filesystems.FileObject)
     * }
     * method. Since this method has package access, new farms cannot be created
     * outside this module! </p>
     *
     * @param farm
     * @param fileObject
     */
    @Override
    public void save(Farm farm) {
        Project project = findFarmProject(farm.getURI());
        FileObject projectRoot = project.getProjectDirectory();
        save(farm, projectRoot);
        if (!projectRoot.getName().equals(farm.getName())) {
            LOGGER.log(Level.FINEST, "Farm name changed to: {0}", farm.getName());

            String actualNewName = FileUtil.findFreeFolderName(projectRoot.getParent(), farm.getName());
            LOGGER.log(Level.FINEST, "Action new free name determined to be: {0}", actualNewName);
//            projectDir.rename(FileLock.NONE, actualNewName, null);
            DefaultProjectOperations.performDefaultRenameOperation(project, actualNewName);
            // update managed farms:
            FarmProjectRegistry.getInstance().add((FarmProject)project);
        }
    }

    @Override
    protected Farm load(AgroURI uri, Project project) {
        assert uri.getItemIdType() == Farm.ITEM_ID_TYPE;

        Farm farm = load(project, project.getProjectDirectory().getFileObject(FarmProject.FARM_FILE));
        if (uri.equals(farm.getURI())) {
            return farm;
        } else {
            return null;
        }
    }

    @Override
    public void remove(Farm farm) {
        Project project = findFarmProject(farm.getURI());
        DefaultProjectOperations.performDefaultDeleteOperation(project);
    }

}
