/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.farm;

import java.awt.event.ActionEvent;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import javax.swing.AbstractAction;
import nl.cloudfarming.client.geoviewer.jxmap.MapDataManager;
import nl.cloudfarming.client.geoviewer.jxmap.MapViewerTopComponent;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import nl.cloudfarming.client.model.Entity;
import nl.cloudfarming.client.model.EntitySavable;
import nl.cloudfarming.client.model.FarmID;
import org.netbeans.api.actions.Savable;
import org.netbeans.api.project.FileOwnerQuery;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ui.OpenProjects;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionReferences;
import org.openide.awt.ActionRegistration;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;
import org.openide.windows.Mode;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;

/**
 * custom close action for farm projects.
 * replaces the default provided project close action.
 * 
 * @author johan
 */
@ActionID(category = "File", id = "nl.cloudfarming.client.farm.FarmProjectCloseAction")
@ActionRegistration(iconBase = "nl/cloudfarming/client/farm/close_project.png",
        displayName = "#CTL_FarmProjectCloseAction")
@ActionReferences({
    @ActionReference(path = "Menu/File", position = 600),
    @ActionReference(path = "FarmProjects/Actions")
})
@NbBundle.Messages({"CTL_FarmProjectCloseAction=Close Farm"})
public class FarmProjectCloseAction extends AbstractAction {
    List<FarmProject> projectsToClose;

    public FarmProjectCloseAction(List<FarmProject> projects) {
        this.projectsToClose = projects;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        Set<FarmID> farmIDs = new HashSet<>();
        Set<EntitySavable> savables = new HashSet<>();
        boolean doClose = true; // can be canceled from save dialog
        
        for (FarmProject project : projectsToClose) {
            farmIDs.add(project.getFarmID());
        }
        
        // collect outstanding changes for the projects to close:
        for (EntitySavable savable : Savable.REGISTRY.lookupAll(EntitySavable.class)) {
            if (farmIDs.contains(savable.getFarmID())) {
                savables.add(savable);
            }
        }
        
        // show save dialog if there are outstanding changes:
        if (!savables.isEmpty()) {
            doClose = FarmProjectSaveDialog.showDialog(savables);
        }
        
        if (doClose) {
            OpenProjects.getDefault().close(projectsToClose.toArray(new FarmProject[projectsToClose.size()]));
            
            // cleanup components containing references to (objects from) the project(s) being closed:
            closeTopComponents();
            removeFromMapViewer();
            // TODO: handle TC's and layerNodes not containing an Entity or DataObject
            // e.g. individual shapes, details from isobus taskdata (planned), ...
            
            // alternatives?
            // - fire a "farm-project closed" event:
            //   cons:
            //   - added dependencies all over the place: eventbus, farm-project, possibly model (FarmID/EIP)
            //   pros:
            //   - extensible: modules (including any future plugins) are responsible for cleaning up instead of the project.
        }
        
    }
    
    
    /**
     * Remove all nodes related to the project(s) being closed from 
     * the map and the layerlist.
     */
    private void removeFromMapViewer() {
        MapDataManager manager = MapViewerTopComponent.findInstance().getMapDataManager();
        
        for (Node node : manager.getLayerNodes()) {
            // look for object types that we can link to a farm project,
            // either by URI or by primary file location:
            Entity entity = node.getLookup().lookup(Entity.class);
            DataObject dataObject = node.getLookup().lookup(DataObject.class);
            
            if (entity != null) {
                for (FarmProject project : projectsToClose) {
                    if (EnterpriseIdProvider.Util.isSameFarm(project.getFarm(), entity.getURI())) {
                        manager.removeLayerNode(node);
                        break;
                    }
                }
            } else if (dataObject != null) {
                FileObject fobj = dataObject.getPrimaryFile();
                Project owner = FileOwnerQuery.getOwner(fobj);
                if (owner instanceof FarmProject && projectsToClose.contains((FarmProject)owner)) {
                    manager.removeLayerNode(node);
                }
            }
        }
    }

    /**
     * Close all topComponents related to the project(s) being closed,
     * which aren't already handled by NetBeans.
     */
    private void closeTopComponents() {
        // based on org.netbeans.modules.project.ui.ProjectUtilities.OPEN_CLOSE_PROJECT_DOCUMENT_IMPL.doClose(),
        // which only handles DataObject based editor TCs
        
        WindowManager wm = WindowManager.getDefault();
        
        for (Mode mode : wm.getModes()) {
            
            // do not skip these, an entity data panel can be moved to a non-editor mode:
            //if (!wm.isEditorMode(mode)) continue;

            for (TopComponent tc : wm.getOpenedTopComponents(mode)) {
                
                // skip the map TC explicitly:
                if (tc instanceof MapViewerTopComponent) {
                    continue;
                }
                
                if (tc.getLookup().lookup(DataObject.class) != null) {
                    // these are already handled by the default NetBeans close mechanism
                    continue;
                }
                
                // FIXME: this might close too much, like logical project tab or map viewer ...
                Entity entity = tc.getLookup().lookup(Entity.class);
                if (entity != null) {
                    for (FarmProject project : projectsToClose) {
                        if (EnterpriseIdProvider.Util.isSameFarm(project.getFarm(), entity)) {
                            tc.close();
                            break;
                        }
                    }
                }
                
            }
        }
        
    }
}
